module InputTable.Messages exposing (..)

{-| This module lists the messages that may be sent by InputTable events


# Definition

@docs TableMsg

-}

import Http
import InputTable.Model exposing (..)
import Set


{-| TableMsg union type contains all messages sent by InputTable events
-}
type TableMsg rowData focussedData
    = SetCellValue ColumnId (rowData -> String -> rowData) RowId Bool String
    | ToggleSideBar (Maybe RowId)
    | SetSideBarTinyMceValue TinyMceResponse
    | SetBoolCellValue ColumnId (rowData -> Bool -> rowData) RowId Bool
    | ToggleCellDropdown RowId ColumnId
    | SelectDropdownParent RowId ColumnId String (rowData -> ( String, Maybe String ) -> rowData)
    | ViewDropdownChildren RowId ColumnId String (rowData -> ( String, Maybe String ) -> rowData)
    | SelectDropdownChild RowId ColumnId String String (rowData -> ( String, Maybe String ) -> rowData)
    | SetSearchText String
    | SetColumnFilterText ColumnId String
    | SwitchColumnCheckboxFilter ColumnId (Maybe Bool)
    | ToggleRowCheckbox RowId
    | ToggleVisibleRowsCheckboxes (List (Row rowData))
    | ToggleVisibleRowsExpansion (List (Row rowData))
    | ToggleChooseVisibleColumnsUi
    | ToggleColumnVisibility ColumnId
    | SortRows (Column rowData)
    | TableClick
    | PreviousPage
    | NextPage Int
    | SetPage String
    | ExportRows
    | ToggleSideTable
    | UpdateSideSearchText String
    | ToggleSideRowCheckbox SideRowId
    | ToggleVisibleSideRowsCheckboxes (Set.Set SideRowId)
    | PostSideRows
    | ToggleAccordion RowId
    | FocusRow (Row rowData)
    | BlurRow
    | FocusOnPrevious
    | FocusOnNext
    | DoNothing


type alias TinyMceResponse =
    { response : String
    , columnId : String
    }
